#include "WProgram.h"
// This example reads out the PixArt PAN3101 Optical Navigation Sensor
// It's used in many cheap optical mouses.
//
// For support for the Agilent ADNS-2051, ADNS-2083 or ADNS-2610, move
// the files for your mouse to the folder with the OptiMouse files.
// Then uncomment the right header files and object instances below.
//
// The Arduino will keep track of a (x, y) coordinate by increasing
// or decreasing the x and y variables by dx and respectively dy.
// Every 128th sample it reports the current (x, y) over the Serial.
//
// Written by Martijn The -> post [at] martijnthe.nl
// Tutorial: http://www.martijnthe.nl/optimouse/
// Based on the sketches by Beno\u00eet Rousseau

// #include "PAN3101.h"
// #include "ADNS2051.h"
// #include "ADNS2610.h"

/******************************************************************************
 *  i2c_gpio
 *  Keith Neufeld
 *  May 26, 2008
 *
 *  Prototype I2C interface to TI 9535 and 9555 GPIO expanders.
 *
 *  Arduino analog input 5 - I2C SCL ===== YELLOW WIRE
 *  Arduino analog input 4 - I2C SDA ===== BLUE WIRE
 *
 ******************************************************************************/




#include "ADNS2083.h"
#include <Wire.h> 

#define SCLK 2                            // Serial clock pin on the Arduino ===== YELLOW WIRE
#define SDIO 3                            // Serial data (I/O) pin on the Arduino  ======BLUE WIRE

void setup();
void loop();
void printer();
void requestEvent();
void killTransaction();
int ledPin = 12;                  // LED test pin
int ledPin2 = 13;                  // LED test pin

// PAN3101 Optical1 = PAN3101(SCLK, SDIO);   // Create an instance of the PAN3101 object
// ADNS2051 Optical1 = ADNS2051(SCLK, SDIO);
// ADNS2610 Optical1 = ADNS2610(SCLK, SDIO);
 ADNS2083 Optical1 = ADNS2083(SCLK, SDIO);

signed long x = 0;                        // Variables for our 'cursor'
signed long y = 0;                        //
char id;

int c = 0;                                // Counter variable for coordinate reporting
float x_cm = 0;                               // Variable that translates the x-coordinate to centimeters
float y_cm = 0;                               // Variable that translates the y-coordinate to centimeters
int sendX = 0;
int sendY = 0;

//debug mode
boolean debug_gps = false;
boolean debug_nxt = true;

//timer variables
unsigned long lastRequest = 0; // needs to hold the number of milliseconds still the program started running
int timePassed = 0;

//protocol variables
int transactionNmr = 0;
boolean alive = false;
boolean confirmed = false;
boolean transfer = false;
boolean dataAvailable = false;

//gps variables
int byteGPS=-1;     // byte containing current received byte

char stringGPS[300] = "";         // String containing the total gps string 
int stringGPSPosition = 0;        // Integer to hold the position of the current current gps character

char stringCheck[7] = "$GPRMC";   // String to check if they gps string is of the type GPRMC (the string that contains actual information)
int stringCheckCounter = 0;

int seperators[13];
int seperatorCounter = 0;          // Integer to store the position of the comma

//GPS Strings
char gps0[7] = "";    //The time in UTC (HhMmSs)
char gps1 = ' ';       //Status of the data (A=active, V=invalid)
char gps2[11] = "";   //Latitude
char gps3 = ' ';       //Latitude Hemisphere (N/S)
char gps4[11] = "";   //Longitude
char gps5 = ' ';       //Longitude Hemisphere (E/W)
char gps6[4] = "";    //Velocity (knots)
char gps7[7] = "";       //Bearing (degrees)
char gps8[4] = "";    //Chec



void setup()
{
  Serial.begin(38400);
  Optical1.begin();                       // Resync (not really necessary?)
  Wire.begin(127);                // join i2c bus with address 127
  Wire.onRequest(requestEvent);   // request event

  for (int i=0;i<300;i++){       // Initialize a buffer for received data
    stringGPS[i]=' ';
  } 

  Serial.println("Arduino booted");
}

void loop()
{

//  The status commands are available only for the PAN3101 and the ADNS2051:

//  Optical1.updateStatus();                // Get the latest motion status
//  if (Optical1.motion())                  // If the 'Motion' status bit is set,
//  {

    x += Optical1.dx();                   // Read the dX register and in/decrease X with that value
    y += Optical1.dy();                   // Same thing for dY register.....
    //id = Optical1.dy();
//  }
}

void printer()
{                                  // Report the coordinates once in a while...
    x_cm = float(x) / 157,48;
    x_cm = 10 * x_cm;
    y_cm = float(y) / 157,48;
    y_cm = 10 * y_cm;
    Serial.print("x = ");
    Serial.print(x_cm, DEC);
    Serial.print(" mm, y = ");
    Serial.print(y_cm, DEC);
    Serial.print(" mm");
    Serial.println();
    sendX = (int)x_cm;
    sendY = (int)y_cm;
    dataAvailable = true;
}

void requestEvent()
{
timePassed = millis() - lastRequest; 
printer();
//Serial.println("hallo");
//first check if we have an alive connection
  if(!alive)
    //a new transaction starts  
  {
    digitalWrite(ledPin2, HIGH);
    alive = true;
    transactionNmr = 0;
    transactionNmr++;
    Wire.send(0);
  }

  else if (alive)
    //a transaction is still running, check which operation to perform based on the delay time
  {
    timePassed = millis() - lastRequest;  

    if(timePassed < 30 && !confirmed)
      //count which transaction to perform
    {
      if(debug_nxt) { 
        Serial.println(timePassed); 
      }
      Wire.send(0);
      transactionNmr++;
      //Serial.println(transactionNmr);
    }

    else if(timePassed >= 40 && timePassed < 60 && !confirmed)
      //the transaction number is received, the NXT wants confirmation
    {
      transactionNmr = transactionNmr - 1;
      Wire.send(transactionNmr);
      confirmed = true;
      if(debug_nxt) { 
        //Serial.println("confirm"); 
      }
    }

    else if(confirmed && alive && !transfer )
      //the actual transaction starts
    {
      //first send character to indicate the transaction is starting
      Wire.send("#");
      transfer = true;
      if(debug_nxt) { 
        //Serial.println("start"); 
      }
    }

    else if(confirmed && alive && transfer)
      //the actual transaction starts
    {
      switch (transactionNmr) {
      case 0:
        if(dataAvailable)
        {
          Serial.println("zenden!");
          Wire.send(sendX);
        }
        else
        {
          Wire.send("error");
          }
        if (debug_nxt) { 
          Serial.println("case 0:"); 
           //Serial.println(sendX);
        }
        break;
      case 1:
        if(dataAvailable)
        {
          Wire.send(sendX);
        }
        else
        {
          Wire.send("error");
        }
        if(debug_nxt) { 
          Serial.println("case 1:"); 
          //Serial.println(sendX); 
        }
        break;
      case 2:
        if(dataAvailable)
        {
          Wire.send(sendY);
        }
        else
        {
          Wire.send("error");
        }
        if(debug_nxt)
        {
          Serial.println("case 2");
         // Serial.println(sendY);
        }
        break;
      case 3:
        if(dataAvailable)
        {
          Wire.send(sendY);
        }
        else
        {
          Wire.send("error");
        }
        if(debug_nxt) { 
          Serial.println("case 3"); 
         // Serial.println(sendY);
        }
        break;
      /*case 4:
        if(dataAvailable)
        {
          Wire.send(gps4);
        }
        else
        {
          Wire.send("error");
        }
        if(debug_nxt)
        {
          Serial.println("case 4");
          Serial.println(sizeof(gps4));
        }
        break;
      case 5:
        if(dataAvailable)
        {
          Wire.send(gps5);
        }
        else
        {
          Wire.send("error");
        }
        if(debug_nxt) { 
          Serial.println("case 5"); 
        } 
        break;  
      case 6:
        if(dataAvailable)
        {
          Wire.send(gps6);
        }
        else
        {
          Wire.send("error");
        }
        if(debug_nxt) { 
          Serial.println("case 6"); 
        }
        break;  
      case 7:
        if(dataAvailable)
        {
          Wire.send(gps7);
        }
        else
        {
          Wire.send("error");
        }
        if(debug_nxt) { 
          Serial.println("case 7"); 
        }
        break;  
      case 8:
        if(dataAvailable)
        {
          Wire.send(gps8);
        }
        else
        {
          Wire.send("error");
        }
        if(debug_nxt) { 
          Serial.println("case 8"); 
        }
        break;*/          
      default: 
        Wire.send("error");
        if(debug_nxt) { 
          Serial.println("else"); 
        }
        break;
      }

      killTransaction();
    }
    else{
      Wire.send(0);

    }

  }
  lastRequest = millis();
}

void killTransaction()
{
  if(debug_nxt) { 
    //Serial.println("transaction stopped"); 
  }
  if(debug_nxt) { 
    Serial.println(" "); 
  }
  digitalWrite(ledPin2, LOW);
  alive = false;
  confirmed = false;
  transfer = false;
  transactionNmr = 0;
}

int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

